searchData={"content_type":"text/markdown","items":[{"doc":"MockMe is a simple mock server used to mock out your third party services in your tests. Unlike many mocking\nsolutions, MockMe starts a real HTTP server and serves real static responses which may be toggled easily using\nthe `MockMe.set_response(:test, :result)` function in your tests.\n\nUnder the hood this package uses [Plug.Router](https://hexdocs.pm/plug/Plug.Router.html) to manage the routes\nand [Plug.Cowboy](https://hexdocs.pm/plug_cowboy/Plug.Cowboy.html) for the HTTP server.\nThe path in the routes can be any valid path accepted by [Plug.Router](https://hexdocs.pm/plug/Plug.Router.html).\nSee the [Plug.Router](https://hexdocs.pm/plug/Plug.Router.html) docs or examples for more information.","ref":"MockMe.html","title":"MockMe","type":"module"},{"doc":"This package can be installed\nby adding `mock_me` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mock_me, \"~> 0.1.0\", only: [:dev, :test]}\n  ]\nend\n```\n\nThe docs can be found at [https://hexdocs.pm/mock_me](https://hexdocs.pm/mock_me).","ref":"MockMe.html#module-installation","title":"Installation - MockMe","type":"module"},{"doc":"Most applications today obtain data from external sources using TCP. Typically, when integrating with these sources\n  you have a few options when writing tests:\n\n  1. Not have automated tests around the code which calls out to these services. Not an option in my opinion.\n  1. Short circuit the code paths before reaching out to the external service using some type of function overwrite mechanism\n  in your tests. While better than not testing, this path often leaves you with untested code paths which could become issues or throw errors later. It also leaves you\n  in a place where your tests do not acurrately document your code.\n  1. Use something like VCR which will make an initial request to the live third party service the first time and then playback that\n  recorded response on subsequent requests. This is a valid strategy, but I've always found it cumbersome to setup and manage. I also like to know\n  exactly what is being returned in requests.\n  1. Use Liskov substitution to replace your API client interface with a mocked out module which mimics the behaviour of your adapter. While this is an excellent way\n  to design your code, and a good idea to ensure your interface contracts, it falls short when doing integration tests because you're not actually testing the code\n  that will be running in production.\n  1. Set up your own mock server which will respond to real HTTP requests and thus test your entire code path just like it would perform in production.\n\n  Of all the options I prefer the last and it's what I do in all my Elixir projects. If you do it from scratch, it's only 2 files and takes very little\n  effort. However, I got tired of setting it up in all my projects so I built an abstraction with simple configuration that will build the server and run\n  it for you in your tests.\n\n  This project is built based on my own personal use. I'm certain there are other use cases and options which you may want to build into it.\n  If you would like to contribute, please head over to the [GitHub Repo](https://github.com/nbriar/mock_me) and request access to make pull requests.\n  I hope you find this project as useful as I have.","ref":"MockMe.html#module-philosophy","title":"Philosophy - MockMe","type":"module"},{"doc":"1. add `{:mock_me, \"~> 0.1.2\", [only: [:test], runtime: false]}` to your dependencies in `mix.exs`\n  1. configure your code to point to the mock server url `http://localhost: `\n  1. configure your routes in your `test/test_helper.exs` file\n  1. start the `MockMe` server in your `/test/test_help.exs` file\n  1. use `MockMe` in your tests","ref":"MockMe.html#module-setup","title":"Setup - MockMe","type":"module"},{"doc":"_config/test.exs_\n\n  ```\n  config :mock_me, port: 9081\n  ```\n\n  This is only used if you want to change the port the mock server listens to. The default port is 9081.","ref":"MockMe.html#module-config","title":"Config - MockMe","type":"module"},{"doc":"Add `:mock_me` to your project dependencies.\n\n_mix.exs_\n\n```\n\n\ndef deps do\n  [\n    {:mock_me, \"~> 0.1.0\"}\n  ]\nend\n  ```","ref":"MockMe.html#module-dependencies","title":"Dependencies - MockMe","type":"module"},{"doc":"_test/test_helpers.ex_\n\n```\n\n\nExUnit.start()\nMockMe.start()\n\nroutes = [\n  %MockMe.Route{\n    name: :swapi_people,\n    path: \"/swapi/people/:id\",\n    responses: [\n      %MockMe.Response{\n        flag: :success,\n        body: MockMePhoenixExample.Test.Mocks.SWAPI.people(:success)\n      },\n      %MockMe.Response{flag: :not_found, body: \"people-failure\", status_code: 404}\n    ]\n  },\n  %MockMe.Route{\n    name: :swapi_starships,\n    path: \"/swapi/starships/:id\",\n    responses: [\n      %MockMe.Response{\n        flag: :success,\n        body: MockMePhoenixExample.Test.Mocks.SWAPI.starships(:success)\n      },\n      %MockMe.Response{flag: :not_found, body: \"starships-failure\", status_code: 404}\n    ]\n  }\n]\n\nMockMe.add_routes(routes)\nMockMe.start_server()\n```","ref":"MockMe.html#module-initialization","title":"Initialization - MockMe","type":"module"},{"doc":"_test/mock_me_phoenix_example/services/starwars.exs_\n\n```\ndefmodule MockMePhoenixExample.Services.StarWarsTest do\n  use ExUnit.Case\n  alias MockMePhoenixExample.Services.StarWars\n\n  # setup_all %{} do\n  #   # re-initializes the test case state\n  #   MockMe.reset_flags()\n  # end\n\n  test \"people/1 returns success\" do\n    MockMe.set_response(:swapi_people, :success)\n    assert {:ok, _} = StarWars.people(1)\n  end\n\n  test \"people/1 returns not found\" do\n    MockMe.set_response(:swapi_people, :not_found)\n    assert {:not_found, _} = StarWars.people(1)\n  end\n\n  test \"starships/1 returns success\" do\n    MockMe.set_response(:swapi_starships, :success)\n    assert {:ok, _} = StarWars.starships(1)\n  end\n\n  test \"starships/1 returns not found\" do\n    MockMe.set_response(:swapi_starships, :not_found)\n    assert {:not_found, _} = StarWars.starships(1)\n  end\nend\n```","ref":"MockMe.html#module-use","title":"Use - MockMe","type":"module"},{"doc":"Add routes to your server. This goes in your `test/test_helper.exs` file.\n\nDefined routes using the `MockMe.Route` struct.\n\nThe first response in the list of responses is considered the default response and will be used\nin the case where you haven't set a flag in your tests or when `reset_flags/0` is called.\n\n#","ref":"MockMe.html#add_routes/1","title":"MockMe.add_routes/1","type":"function"},{"doc":"_test/test_helper.exs_\n```\nroute = %Route{\n  name: :test_me,\n  path: \"/jwt\",\n  responses: [\n    %Response{flag: :success, body: \"some-body\"}\n  ]\n}\n\nMockMe.start()\nMockMe.add_routes([route])\nMockMe.start_server()\n```","ref":"MockMe.html#add_routes/1-example","title":"Example - MockMe.add_routes/1","type":"function"},{"doc":"Called inside each endoint to determine which response to return.\nYou should never need to call this in your code except in the case of troubleshooting.","ref":"MockMe.html#flag_value/1","title":"MockMe.flag_value/1","type":"function"},{"doc":"A convienience function to view the state of the mocks.\nPrimarily used for troubleshooting. You shouldn't need this in any of your tests.","ref":"MockMe.html#get_state/0","title":"MockMe.get_state/0","type":"function"},{"doc":"Used to reset the test state to the config defaults once tests in a module have been performed or before tests are run.\n\nUse this in the `setup` block of your tests or inside a test which you need to be sure uses the defaults.\n\n```\nsetup do\n  MockMe.reset_flags()\nend\n```","ref":"MockMe.html#reset_flags/0","title":"MockMe.reset_flags/0","type":"function"},{"doc":"Used to get the defined routes from state.","ref":"MockMe.html#routes/0","title":"MockMe.routes/0","type":"function"},{"doc":"The primary function in your tests. Call this to toggle responses from the mock server.\n\nTo use this in your tests you can call:\n\n`MockMe.set_response(:route_name, :route_flag)`\n\nThe response with the defined `:flag` will be returned when the endpoint is called.","ref":"MockMe.html#set_response/2","title":"MockMe.set_response/2","type":"function"},{"doc":"Start the application state in the unit tests.\nThis prepares the state to accept the configuration for your mocked routes.\nTo add routes use the `MockMe.add_routes/1` functions.","ref":"MockMe.html#start/0","title":"MockMe.start/0","type":"function"},{"doc":"Used to start the mock server after routes have been added to state using `add_routes/1`","ref":"MockMe.html#start_server/0","title":"MockMe.start_server/0","type":"function"},{"doc":"Used to define how the mocked endpoint should respond.","ref":"MockMe.Response.html","title":"MockMe.Response","type":"module"},{"doc":"```\n  %Response{\n    flag: :success,               #required\n    body: \"some-serialized-body\", #required\n    status_code: 200,             #default\n    headers: [],                  #default\n    cookies: [],                  #default\n  }\n  ```\n\n  `headers` is an array of tuples with the header and value: `{\"content-type\", \"application/json\"}`\n  `cookies` is an array of tuples with the name, value and options which follow the signature of `Plug.Conn.set_resp_cookie`\n  `{\"my-cookie\", %{user_id: user.id}, sign: true}`","ref":"MockMe.Response.html#module-example","title":"Example - MockMe.Response","type":"module"},{"doc":"Used to handle the toggling of the responses based on the route flag.","ref":"MockMe.ResponsePlug.html","title":"MockMe.ResponsePlug","type":"module"},{"doc":"","ref":"MockMe.ResponsePlug.html#call/2","title":"MockMe.ResponsePlug.call/2","type":"function"},{"doc":"","ref":"MockMe.ResponsePlug.html#init/1","title":"MockMe.ResponsePlug.init/1","type":"function"},{"doc":"","ref":"MockMe.ResponsePlug.html#set_response_cookies/2","title":"MockMe.ResponsePlug.set_response_cookies/2","type":"function"},{"doc":"","ref":"MockMe.ResponsePlug.html#set_response_headers/2","title":"MockMe.ResponsePlug.set_response_headers/2","type":"function"},{"doc":"A struct for defining routes and their test cases","ref":"MockMe.Route.html","title":"MockMe.Route","type":"module"},{"doc":"```\n  %Route{\n    name: :my_test_case,              #required\n    path: \"/my/test/case\",            #required\n    method: :get,                     # default\n    content-type: \"application/json\", #default\n    responses: [],                    #default\n  }\n  ```\n\n  `name:` must be an atom which you use to identify this route in your tests\n\n  `path:` must be a valid http url - usually the route for the api you're mocking\n\n  `method:` may be either a single atom or a list of atoms - valid atoms are :get, :post, :put, :patch, :delete and :options\n\n  `content-type:` must be a valid http response type, this is the content-type header for the response\n\n  `responses:` must be an empty list or a list of %MockMe.Response{}","ref":"MockMe.Route.html#module-example","title":"Example - MockMe.Route","type":"module"},{"doc":"Used to keep track of state for mocks in tests.\nHolds a map of route names and response flags which the server uses to determine which response to serve.","ref":"MockMe.State.html","title":"MockMe.State","type":"module"},{"doc":"```\n%{\n  routes: [\n    %MockMe.Route{\n      name: :test_me,\n      path: \"/test-path\",\n      responses: [\n        %MockMe.Response{flag: :success, body: \"test-body\"},\n        %MockMe.Response{flag: :failure, body: \"test-failure-body\"}\n      ]\n    }\n  ],\n  cases: %{\n    test_me: :success\n  }\n}\n```\nThese values are populated from `MockMe.add_routes/1` and then toggled using `MockMe.set_response(:route_name, :route_flag)`.","ref":"MockMe.State.html#module-example","title":"Example - MockMe.State","type":"module"},{"doc":"","ref":"MockMe.State.html#child_spec/1","title":"MockMe.State.child_spec/1","type":"function"},{"doc":"","ref":"MockMe.State.html#start_link/1","title":"MockMe.State.start_link/1","type":"function"}]}